@@ -3,12 +3,21 @@ use strict;
 use warnings;
 use JSON qw//;
 use Sub::Data::Recursive;
+use POSIX qw/strftime/;
 use Getopt::Long qw/GetOptionsFromArray/;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 my $MAX_DEPTH = 10;
 
+my $MAYBE_UNIXTIME = join '|', (
+    '.ed(?:_at|_on)?',
+    'date',
+    'time',
+);
+
+my $UNIXTIMESTAMP_KEY = '';
+
 sub new {
     my $class = shift;
     my @argv  = @_;
@@ -50,8 +59,10 @@ sub process {
     }
     else {
         Sub::Data::Recursive->invoke(\&_split => $decoded) if $self->opt('x');
+        Sub::Data::Recursive->invoke(\&_more_split => $decoded) if $self->opt('xx');
         $self->{_depth} = $self->opt('depth');
         $self->_recursive_decode_json($decoded);
+        Sub::Data::Recursive->massive_invoke(\&_convert_timestamp => $decoded) if $self->opt('xxx');
         return $self->{_json}->encode($decoded);
     }
 }
@@ -66,6 +77,43 @@ sub _split {
     }
 }
 
+sub _more_split {
+    my $line = $_[0];
+
+    chomp $line;
+
+    $line =~ s!([])>])\s+([[(<])!$1$2!g;
+    $line =~ s!([[(<][^])>]+[])>])!$1\n!g; # '\n' already replaced by --x option
+    my @element = split /\n/, $line;
+
+    $_[0] = \@element;
+}
+
+sub _convert_timestamp {
+    my $line    = $_[0];
+    my $context = $_[1];
+    my $keys    = $_[2];
+
+    return if !$context || $context ne 'HASH';
+
+    if (my $key = $keys->[0]) {
+        if (
+            ($UNIXTIMESTAMP_KEY && $key eq $UNIXTIMESTAMP_KEY && $line =~ m!(\d+(\.\d+)?)!)
+                || ($key =~ m!(?:$MAYBE_UNIXTIME)!i && $line =~ m!(\d+(\.\d+)?)!)
+        ) {
+            my $unix_timestamp = $1;
+            my $msec = $2 || '';
+            if ($unix_timestamp > 2**31 -1) {
+                ($msec) = ($unix_timestamp =~ m!(\d\d\d)$!);
+                $msec = ".$msec";
+                $unix_timestamp = int($unix_timestamp / 1000);
+            }
+            my $date = strftime('%c', localtime($unix_timestamp)) . $msec;
+            $_[0] = "$line ($date)";
+        }
+    }
+}
+
 sub _recursive_decode_json {
     my ($self, $hash) = @_;
 
@@ -94,6 +142,9 @@ sub _parse_opt {
         'depth=s'   => \$opt->{depth},
         'no-pretty' => \$opt->{no_pretty},
         'x'         => \$opt->{x},
+        'xx'        => \$opt->{xx},
+        'xxx'       => \$opt->{xxx},
+        'timestamp-key=s' => \$opt->{timestamp_key},
         'h|help'    => sub {
             $class->_show_usage(1);
         },
@@ -105,6 +156,11 @@ sub _parse_opt {
 
     $opt->{depth} ||= $MAX_DEPTH;
 
+    $opt->{x}  ||= $opt->{xx} || $opt->{xxx};
+    $opt->{xx} ||= $opt->{xxx};
+
+    $UNIXTIMESTAMP_KEY = $opt->{timestamp_key};
+
     return $opt;
 }
 
@@ -161,7 +217,7 @@ The parser of the line
 
 =begin html
 
-<a href="http://travis-ci.org/bayashi/App-jl"><img src="https://secure.travis-ci.org/bayashi/App-jl.png?_t=1557473289"/></a> <a href="https://coveralls.io/r/bayashi/App-jl"><img src="https://coveralls.io/repos/bayashi/App-jl/badge.png?_t=1557473289&branch=master"/></a>
+<a href="http://travis-ci.org/bayashi/App-jl"><img src="https://secure.travis-ci.org/bayashi/App-jl.png?_t=1560057096"/></a> <a href="https://coveralls.io/r/bayashi/App-jl"><img src="https://coveralls.io/repos/bayashi/App-jl/badge.png?_t=1560057096&branch=master"/></a>
 
 =end html
 
